function [Y,rec,obj]=voc_optimize(P,options);
%VOC_OPTIMIZE -- Optimize VOC utility function given marginal posteriors.
%
% Input
%    P: (single or double),
%       either
%          (class_count,N) array of posterior marginals so that for any i we
%             have sum(P(:,i))==1, and X(c,i) >= 0;
%       or
%          (1,M) cellarray of (class_count,h_j,w_j) posterior maps.
%       In both cases class_count needs to be smaller or equal to 255.
%
%    options: (optional) structure with the following elements,
%       .init: type of initialization used, either 'random' (default), 'map',
%          or 'inverseweighted'.
%          If options.Yinit is provided, the options.init option cannot be
%          used.
%       .algorithm: 'lfp' or 'greedy', default is 'lfp'.
%       .sweep_count: number of relabel sweeps, default: 30.
%       .subset_size: number of variables in a subset, or zero if all
%          variables are optimized, default: 0.
%          Reasonable values range from 100-10000.  Use this in case
%          optimizing over all variables is too slow.
%       .Yinit: same format as Y (so either a (1,N) uint8 vector or an M
%          element cellarray of uint8 arrays), used as initial solution.
%
% Output
%    Y: If the first type of P argument has been provided, a (1,N) uint8
%       class.  The elements are >= 1, <= class_count.
%       If the secodn type of P argument has been given we return a (1,M)
%       cellarray of (h_j,w_j) optimal decision maps with elements >= 1 and
%       <= class_count.
%    rec: (optional), iteration information, (2,iterations) array with each
%       column containing [obj; time].
%    obj: (1,class_count) per-class objective of returned solution.  The total
%       objective function is the average, mean(obj).
%
% Author: Sebastian Nowozin <Sebastian.Nowozin@microsoft.com>
% Date: 17th October 2013.

if nargin < 2
	options=[];
end
try
	[Y,rec,obj]=mex_voc_optimize(P,options);
catch err
	error('Could not call mex_voc_optimize, did you compile the mex file using compile_windows.m or compile_linux.m?');
end

