
#include <iostream>
#include <fstream>
#include <random>

#include "lfp.h"
#include "timing.h"

#ifdef PROFILE
#include <google/profiler.h>
#endif

typedef lfp::problem::vector vector;

void test1(void) {
	double A0 = 0.0;
	vector A { 0.141255554318601, 0.045690378127992, 0.172384627427200,
		0.198016283750705, 0.129637051279823 };
	double B0 = 2.485389362752176e+002;
	vector B { 0.858744445681399, 0.954309621872008, 0.827615372572800,
		0.801983716249295, 0.870362948720177 };
	double C0 = 4.463442872281251;
	vector C { -0.011851576562872, -0.006627161275112, -0.013781029382527,
		-0.003865167456107, -0.008576647226181 };
	double D0 = 3.953376615787923e+002;
	vector D { -0.988148423437128, -0.993372838724888, -0.986218970617473,
		-0.996134832543893, -0.991423352773819 };

	lfp::problem prob(A0, A, B0, B, C0, C, D0, D);
	std::cout << prob;

	std::vector<double> sol;

	prob.solve(sol);
}

void test_file(const std::string& filename, bool verify = false) {
	std::ifstream ifs(filename);
	size_t n;
	ifs >> n;
	double A0;
	ifs >> A0;
	vector A(n, 0.0);
	for (size_t i = 0; i < n; ++i) {
		ifs >> A[i];
	}
	double B0;
	ifs >> B0;
	vector B(n, 0.0);
	for (size_t i = 0; i < n; ++i) {
		ifs >> B[i];
	}
	double C0;
	ifs >> C0;
	vector C(n, 0.0);
	for (size_t i = 0; i < n; ++i) {
		ifs >> C[i];
	}
	double D0;
	ifs >> D0;
	vector D(n, 0.0);
	for (size_t i = 0; i < n; ++i) {
		ifs >> D[i];
	}
	ifs.close();

	lfp::problem prob(A0, A, B0, B, C0, C, D0, D);
	assert(prob.check_instance_ok());
#if 0
	std::cout << prob;
#endif

	std::vector<double> sol;
	auto stat = prob.solve(sol);
	std::cout << stat;
	std::cout << std::endl;
	if (verify == false)
		return;

#if 0
	for (size_t i = 0; i < sol.size(); ++i) {
		std::cout << "z_" << i << " " << sol[i] << std::endl;
	}
#endif

	// Verify against ground truth solution obtained via Matlab implementation
	// This is for instance1.txt
	double obj_opt = 0.097204331917804;
	std::vector<double> sol_opt = {
		1, 0, 1, 1, 1, 0, 1, 1, 1, 0, 1, 0, 1, 0, 1, 1, 0, 1, 1, 1,
		1, 1, 1, 1, 1, 0, 1, 0, 0, 0, 1, 1, 1, 1, 0, 1, 1, 1, 1, 1,
		1, 1, 1, 1, 1, 1, 0, 1, 1, 1, 1, 1, 0, 0, 0, 1, 0, 1, 1, 0,
		0, 0, 1, 1, 0, 1, 1, 1, 1, 0, 1, 1, 1, 0, 1, 1, 0, 1, 1, 1,
		1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 0, 0, 1, 1, 1, 0, 1, 1, 1, 1,
		0, 1, 1, 0, 0, 0, 1, 1, 1, 1, 0, 1, 1, 1, 1, 1, 1, 0, 1, 1,
		0, 1, 0, 0, 0, 0, 1, 0, 1, 1, 1, 1, 1, 0, 1, 1, 1, 1, 1, 0,
		1, 0, 1, 0, 0, 1, 1, 1, 1, 1, 1, 1, 1, 1, 0, 1, 1, 1, 1, 0,
		1, 1, 1, 1, 1, 1, 0, 1, 1, 1, 1, 1, 1, 0, 1, 1, 1, 1, 1, 1,
		0, 1, 0, 0, 1, 0, 1, 1, 0, 1, 1, 1, 1, 0, 1, 1, 1, 1, 1, 0,
		0, 0, 1, 1, 0, 1, 1, 0, 1, 0, 0, 1, 0, 0, 1, 1, 0, 1, 1, 1,
		1, 0, 1, 1, 1, 1, 0, 0, 0, 1, 1, 1, 1, 1, 1, 1, 1, 1, 0, 1,
		1, 0, 1, 1, 1, 1, 1, 1, 0, 1, 0, 1, 0, 0, 0, 1, 0, 1, 1, 0,
		1, 0, 1, 1, 1, 0, 1, 1, 0, 1, 1, 1, 0, 1, 0, 1, 1, 1, 0, 1,
		1, 1, 0, 1, 0, 1, 1, 0, 1, 1, 0, 1, 1, 0, 1, 0, 1, 1, 0, 1,
		1, 1, 1, 0, 0, 0, 0, 0, 1, 1, 1, 0, 1, 0, 1, 1, 1, 1, 1, 0,
		1, 1, 1, 1, 1, 1, 1, 1, 0, 1, 1, 1, 1, 1, 0, 0, 0, 0, 1, 1,
		1, 1, 1, 1, 1, 0, 1, 1, 1, 1, 1, 0, 1, 0, 1, 0, 1, 0, 1, 1,
		0, 1, 1, 1, 1, 0, 0, 1, 1, 1, 0, 0, 1, 1, 1,
	};
	assert(std::fabs(obj_opt - stat.f_xi_opt) <= 1.0e-9);
	assert(sol.size() == sol_opt.size());
	for (size_t i = 0; i < sol.size(); ++i) {
		assert(std::fabs(sol_opt[i] - sol[i]) <= 1.0e-9);
	}
}

void test_random(size_t n, unsigned long seed = 0) {
	std::random_device gseed;
	std::mt19937 eng(seed == 0 ? gseed() : seed);
	std::uniform_real_distribution<double> runi;

	double A0 = 0.5 * static_cast<double>(n) * runi(eng);
	vector A(n, 0.0);
	double B0 = 0.0;
	vector B(n, 0.0);
	double C0 = 0.0;
	vector C(n, 0.0);
	double D0 = n;
	vector D(n, 0.0);

	for (size_t i = 0; i < n; ++i) {
		double p1 = runi(eng);
		double p2 = runi(eng);
		double cs = runi(eng)/(p1+p2);
		p1 *= cs;
		p2 *= cs;

		A[i] = p1;
		B0 += p1;
		B[i] = 1.0 - p1;
		C0 += p2;
		C[i] = -p2;
		D[i] = p2-1.0;
	}
	lfp::problem prob(A0, A, B0, B, C0, C, D0, D);

	std::vector<double> sol;

	timer t;
	t.start();
	auto stat = prob.solve(sol);
	t.stop();
	std::cout << stat;
	std::cout << std::endl;
#if 0
	for (size_t i = 0; i < sol.size(); ++i) {
		std::cout << sol[i] << " " << std::endl;
	}
#endif
}

int main(int argc, char* argv[]) {
#if 1
	test1();
	test_file("instance1.txt", true);
	test_file("instance2.txt", false);
	test_file("instance3.txt");
	test_file("instance4.txt");
	test_file("instance5.txt");
#endif

#if 1
#ifdef PROFILE
	ProfilerStart("/tmp/prof3");
#endif
#if 1
	for (size_t ri = 0; ri < 1000; ++ri)
		test_random(500, ri);
#endif
	test_random(15000);
#ifdef PROFILE
	ProfilerStop();
#endif
#endif
}

