
#include <vector>
#include <utility>
#include <cassert>

namespace voc {

class pairpartition {
private:
	size_t class_count;

	typedef std::pair<size_t, size_t> classpair_t;
	typedef std::vector<classpair_t> indep_set_t;
	std::vector<indep_set_t> sets;

	void create_pair_partition(void) {
		sets.clear();
		std::vector<bool> processed((class_count*(class_count-1))/2, false);

		while (true) {
			// Create a new independent set, greedily
			std::vector<bool> curset_classes_used(class_count, false);
			indep_set_t curset;

			size_t k_idx = 0;
			for (size_t k1 = 0; k1 < class_count; ++k1) {
				for (size_t k2 = k1+1; k2 < class_count; ++k2) {
					// Already processed this pair
					if (processed[k_idx]) {
						k_idx += 1;
						continue;
					}

					// Cannot use this pair in the current set, conflict
					if (curset_classes_used[k1] || curset_classes_used[k2]) {
						k_idx += 1;
						continue;
					}

					// Add to set
					curset.push_back(classpair_t(k1, k2));
					curset_classes_used[k1] = true;
					curset_classes_used[k2] = true;
					processed[k_idx] = true;
					k_idx += 1;
				}
			}
			if (curset.empty())
				break;

			sets.push_back(curset);
		}
	}

public:
	pairpartition(size_t class_count)
		: class_count(class_count)
	{
		create_pair_partition();
	}

	size_t number_of_sets(void) const {
		return sets.size();
	}

	const indep_set_t& get_set(size_t idx) const {
		assert(idx < sets.size());
		return sets[idx];
	}
};

}

