
#ifndef TIMING_H
#define TIMING_H

#include <chrono>
#include <functional>
#include <string>
#include <iostream>
#include <cassert>

class timer {
private:
	bool running;
	double c_elapsed_ms;
	std::chrono::time_point<std::chrono::high_resolution_clock> start_time;

	double elapsed_this_period(void) const {
		assert(running);

		std::chrono::time_point<std::chrono::high_resolution_clock> end_time
			= std::chrono::high_resolution_clock::now();
		return static_cast<double>(std::chrono::duration_cast<
			std::chrono::milliseconds>(end_time-start_time).count());
	}

public:
	timer(void) : running(false), c_elapsed_ms(0) {
	}

	void start(void) {
		start_time = std::chrono::high_resolution_clock::now();
		running = true;
	}
	void stop(void) {
		c_elapsed_ms += elapsed_this_period();
		running = false;
	}
	double elapsed_ms(void) const {
		if (running)
			return c_elapsed_ms + elapsed_this_period();

		// Not running
		return c_elapsed_ms;
	}
	double elapsed_s(void) const {
		return elapsed_ms() / 1000.0;
	}
};

class scoped_timer {
private:
	std::string str;
	timer t;

public:
	scoped_timer(const std::string& str) : str(str) {
		t.start();
	}
	~scoped_timer(void) {
		std::cout << "[" << str << "]  " << t.elapsed_ms() << "ms" << std::endl;
	}
};

#endif

