
#include <iostream>
#include <iomanip>
#include <fstream>
#include <random>
#include <cmath>
#include <cassert>

#include "voc.h"
#include "dirichlet.h"

void sim(double dir_alpha, size_t n, size_t nstep, size_t class_count,
	unsigned long seed = 0)
{
	std::random_device gseed;
	std::mt19937 eng(seed == 0 ? gseed() : seed);
	std::uniform_real_distribution<float> runi;

	dirichlet<double> dir(class_count, dir_alpha);

	std::vector<double> P(n*class_count);
	std::vector<double> prior(class_count, 0.0);
	for (size_t i = 0; i < n; ++i) {
		dir.simulate(prior, eng);
		for (size_t c = 0; c < class_count; ++c)
			P[i*class_count + c] = prior[c];
	}

	size_t mc_count = 100000;
	std::cout << std::fixed << std::setprecision(std::numeric_limits<double>::digits10);
	for (size_t ncur = nstep; ncur <= n; ncur += nstep) {
		voc::prediction_function<double> pred(&P[0], class_count, ncur);

		auto iou = pred.compute_iou_c_montecarlo(0, mc_count);
		double EST = std::get<0>(iou);
		double EST_approx = std::get<1>(iou) / std::get<2>(iou);
		std::cout << ncur << " " << EST << " " << EST_approx
			<< " " << (EST-EST_approx) << std::endl;
	}
}

int main(int argc, const char* argv[]) {
	//sim(0.2, 1000, 25, 2);
	sim(0.2, 5000, 100, 2);
}

